#include <unistd.h>
#include <time.h>

//Include X11 window related headers
#include <X11/Xlib.h>
#include <X11/Xutil.h>

// Includes
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// Helper functions (CUDA related)
// CUDA system and GL includes
#include <cuda_runtime.h>
#include <device_types.h>

// Helper functions
#include <helper_functions.h> // CUDA SDK Helper functions
#include <helper_cuda.h>      // CUDA device initialization helper functions

#define DEFAULT_IMAGE_SIZE_WIDTH 1280
#define DEFAULT_IMAGE_SIZE_HEIGHT 720

#define INPUT_FILENAME "out.data"
#define OUTPUT_FILENAME_TEMPLATE "result_%d.data"

typedef unsigned char uchar;


extern "C"
void initTexture(int nWidth, int nHeight, uchar * pData);
extern "C"
void cudaConvertBG10ToRGB(int nWidth, int nHeight,
		dim3 blockSize, dim3 gridSize, uchar4 * pOutput);

int main(int argc, char ** argv) {

	cudaDeviceProp cudaProps;
	int nDevID =findCudaDevice(argc, (const char**) argv);

	checkCudaErrors(cudaGetDeviceProperties(&cudaProps, nDevID));

	printf("Current CUDA device [%s] processor count [%d]\n",cudaProps.name, cudaProps.multiProcessorCount);

	uchar4 * pOutput;
	checkCudaErrors(cudaMalloc((void **)&pOutput, DEFAULT_IMAGE_SIZE_WIDTH * DEFAULT_IMAGE_SIZE_HEIGHT * 4));
	dim3 blockSize(16,16);
	dim3 gridSize(DEFAULT_IMAGE_SIZE_WIDTH/blockSize.x, DEFAULT_IMAGE_SIZE_HEIGHT/blockSize.y);

	char *  pDataBuffer = (char*)malloc(DEFAULT_IMAGE_SIZE_WIDTH * DEFAULT_IMAGE_SIZE_HEIGHT * 4);
	char *  pTempBuffer = (char*)malloc(DEFAULT_IMAGE_SIZE_WIDTH * DEFAULT_IMAGE_SIZE_HEIGHT * 4);

	FILE * pImageFile;
	pImageFile = fopen("out.data", "rb");
	if (pImageFile) {
		fread(pDataBuffer, sizeof (ushort), DEFAULT_IMAGE_SIZE_WIDTH * DEFAULT_IMAGE_SIZE_HEIGHT, pImageFile);
		fclose(pImageFile);
		pImageFile = 0;
	} else {
		printf("cannot read image data file.\n");
		return 0;
	}

	int nCount = 0;

	do {
		struct timeval  tv;
		gettimeofday(&tv, NULL);
		uint32_t oldtime = (tv.tv_sec) * 1000 + (tv.tv_usec) / 1000;

		initTexture(DEFAULT_IMAGE_SIZE_WIDTH, DEFAULT_IMAGE_SIZE_HEIGHT, (uchar*)pDataBuffer);
		cudaConvertBG10ToRGB(DEFAULT_IMAGE_SIZE_WIDTH, DEFAULT_IMAGE_SIZE_HEIGHT,blockSize, gridSize, pOutput);
		checkCudaErrors(cudaDeviceSynchronize());
		cudaMemcpy(pTempBuffer, pOutput, DEFAULT_IMAGE_SIZE_HEIGHT * DEFAULT_IMAGE_SIZE_WIDTH * 4, cudaMemcpyDeviceToHost);
		gettimeofday(&tv, NULL);
		uint32_t newtime =  (tv.tv_sec)*1000 + (tv.tv_usec) / 1000 ; //
		printf("result time: %d\r\n", newtime-oldtime);

		char * pFilename = (char *) malloc(32 * sizeof(char));
		sprintf(pFilename,OUTPUT_FILENAME_TEMPLATE, nCount);
		pImageFile  = fopen(pFilename, "wb");
		if (pImageFile) {
			fwrite(pTempBuffer, sizeof(int), DEFAULT_IMAGE_SIZE_WIDTH * DEFAULT_IMAGE_SIZE_HEIGHT,pImageFile);
			fflush(pImageFile);
			fclose(pImageFile);
			pImageFile = 0;
		}

	}while (++nCount < 10);

	free(pDataBuffer);
	checkCudaErrors(cudaFree(pOutput));

	return 0;
}
